package network;

//import agent.network.CAgent;

import com.sun.messaging.ConnectionConfiguration;
import com.sun.messaging.ConnectionFactory;

import javax.jms.Connection;
import javax.jms.JMSException;
import javax.jms.Session;

/**
 * 
 * Create a JMS communicator class Model for implementing communication (With
 * Queue and Topics)
 * 
 * 
 */
public abstract class CAbstractJMSCommunicator {

	/* JMS Connection factory */
	protected ConnectionFactory itsConnectionFactory = null;
	/* JmsConnection */
	protected Connection itsConn = null;
	/* JMS Abstract Session */
	protected Session itsJMSSession;

	protected boolean itsConnected = false;

	/* Broker IP and Port */
	protected String itsServerIP;
	protected String itsServerPort;

	/* Thread for attempt to reconnect when connection is unavailable */
	private Thread itsReconnector;
	
	
	/* Destination Name of the JMS resource managed by Daughter Class object */
	private String itsDestResourceName;

	/**
	 * Create a JMS persistant connexion to the server
	 */
	protected boolean createPersistentConnection(String theJMSResourceJndiName) {
		try {

			// Store the Destination resource Name
			itsDestResourceName = theJMSResourceJndiName;

			// itsConnectionFactory = (ConnectionFactory) objX;
			itsConnectionFactory = new ConnectionFactory();
			itsConnectionFactory.setProperty(
					ConnectionConfiguration.imqReconnectEnabled, "true");
			itsConnectionFactory.setProperty(
					ConnectionConfiguration.imqBrokerHostName, itsServerIP);
			itsConnectionFactory.setProperty(
					ConnectionConfiguration.imqBrokerHostPort, itsServerPort);
			
			// get a connection
			itsConn = itsConnectionFactory.createConnection();

			// Create new session for messaging
			itsJMSSession = itsConn.createSession(false,Session.AUTO_ACKNOWLEDGE);

			itsConn.start();

		} catch (JMSException ex) {
			
			System.out
					.println("CAbstractJMSCommunicator.createPersistentConnection: "
							+ ex.getMessage());
			return false;
		}
	
		return true;

	}

	/**
	 * Close the persistant connexion
	 */
	protected void closePersistantConnection() {
		try {
			itsConn.stop();
			// Close The Session
			itsJMSSession.close();
			// Close the connection
			itsConn.close();
		} catch (JMSException ex) {
			System.out.println("CAbstractJMSCommunicator.closePersistantConnection: "+ ex.getMessage());
		}
	}

	/**
	 * Stop the communication
	 */
	public void stop() {
		closePersistantConnection();
	}

	
	public void startReconnectThread(){
		if (itsReconnector == null) {
			itsReconnector = new Thread(new Runnable() {
				public void run() {
					while (itsReconnector != null) {
						try {
							synchronized (this) {
								this.wait(5000);
							}
							if (createPersistentConnection(itsDestResourceName))
								itsReconnector = null;
						} catch (InterruptedException ex) {
							System.out.println("CAbstractJMSCommunicator.handlesJMSExceptions: "+ ex.getMessage());
						} catch (Exception ex) {
							System.out.println("CAbstractJMSCommunicator.handlesJMSExceptions: "+ ex.getMessage());
						}
					}
				}
			});
			// Start the new try of connection
			itsReconnector.start();
		}
	}

	
	@Override
	protected void finalize() throws Throwable {
		stop();
		super.finalize();
	}


}
